import typing
from typing_extensions import Annotated
from .client import ProjectsClient, AsyncProjectsClient
from pydantic import model_validator, validator, Field, ConfigDict
from label_studio_sdk._extensions.pager_ext import SyncPagerExt, AsyncPagerExt, T
from label_studio_sdk.types.project import Project
from label_studio_sdk.label_interface import LabelInterface

from ..core import RequestOptions


class ProjectExt(Project):

    def get_label_interface(self):
        return LabelInterface(self.label_config)


class ProjectsClientExt(ProjectsClient):

    def list(self, **kwargs) -> SyncPagerExt[T]:
        return SyncPagerExt.from_sync_pager(super().list(**kwargs))

    list.__doc__ = ProjectsClient.list.__doc__

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> ProjectExt:
        return ProjectExt(**dict(super().get(id, request_options=request_options)))


class AsyncProjectsClientExt(AsyncProjectsClient):

    async def list(self, **kwargs):
        return await AsyncPagerExt.from_async_pager(await super().list(**kwargs))

    list.__doc__ = AsyncProjectsClient.list.__doc__



"""
"""
from typing import Dict, Optional, List, Tuple, Any, Callable, Union
import xml.etree.ElementTree as ET
from xml.dom import minidom

from label_studio_sdk.label_interface.base import LabelStudioTag

import label_studio_sdk.label_interface.object_tags as OT
import label_studio_sdk.label_interface.control_tags as CT

## syntatic sugar helper functions

def labels(labels, tag_type="Labels", **kwargs):
    """
    Converts labels to a tuple structure.
    
    Args:
        labels: The labels to be converted.
        tag_type: Tag type to assign to each label.
        
    Returns:
        A tuple containing labels as per the tag type.
    """
    return (tag_type, kwargs, _convert_to_tuple(labels, tag_type="Label"))
    
    
def taxonomy(labels, **kwargs):
    """
    Constructs a taxonomy.
    
    Args:
        labels: The labels to construct the taxonomy.
        
    Returns:
        A tuple containing the taxonomy.
    """
    result = []
    
    for arg in labels:
        if isinstance(arg, tuple):
            parent, *children = _convert_to_tuple(arg)
            parent_children = (parent[0], parent[1], tuple(children))
            result.append(parent_children)            
        else:
            result.append(("Choice", {"value": arg}, {}))
    
    return ("Taxonomy", kwargs, tuple(result))    

def choices(labels, **kwargs):
    """
    Constructs a choices tuple.
    
    Args:
        labels: The labels to be converted.
        
    Returns:
        A tuple containing the choices.
    """ 
    return ("Choices", kwargs, _convert_to_tuple(labels))


def _prettify(element):
    """
    Returns a pretty-printed XML string for the Element.
    """
    rough_string = ET.tostring(element, 'utf-8')
    reparsed = minidom.parseString(rough_string)
    return reparsed.toprettyxml(indent="  ")


def _convert_to_tuple(args, tag_type="Choice"):
    """
    Converts arguments to a tuple structure of choices.

    Args:
        args: The labels to be converted.
        tag_type: Tag type to assign to each label.

    Returns:
        A tuple containing all labels in specified format.
    """
    if not args:
        return None
    
    return tuple(((tag_type, {"value": arg}, {})) for arg in args)


def _create_nested_elements(parent: ET.Element, elements: List[Union[str, Dict[str, str], List]]) -> None:
    """
    Adds nested elements to the parent element in the Element Tree.

    Args:
        parent: The parent ET.Element object.
        elements: A list containing element definitions.
    """    
    for element in elements:        
        if isinstance(element, str):
            ET.SubElement(parent, element)
        elif isinstance(element, dict):
            for k, v in element.items():
                parent.set(k, v)
        elif isinstance(element, list) or isinstance(element, tuple):
            create_element(element, parent=parent)

            
def _convert(name: str, tag: Union[str, list, tuple, LabelStudioTag]) -> tuple:
    """
    Converts tags from str, list, tuple, or class instance to a tuple format.

    Args:
        name: Name of the tag.
        tag: Tag in str, list, tuple or class instance format.

    Returns:
        A tuple version of the input tag.
    
    Raises:
        TypeError: If input tag is not a str, list, tuple, or LabelStudioTag.
    """
    
    if isinstance(tag, LabelStudioTag):
        tag.name = tag.name or name
        child_tag_type = "Choice" if tag.tag in ["Choices", "Taxonomy"] else "Label"
        el = tag.tag, tag.collect_attrs(), _convert_to_tuple(getattr(tag, "labels", None), tag_type=child_tag_type)
    elif isinstance(tag, (list, tuple)):
        el = (*tag, ()) if len(tag) < 3 else tag
    elif isinstance(tag, str):        
        el = tag, {}, ()
    else:
        raise TypeError("Input tag must be one of str, list, tuple, LabelStudioTag")

    el[1].setdefault("name", name)
    
    if el[0].lower() in OT._TAG_TO_CLASS and not el[1].get("value"):
        el[1]["value"] = "$" + name
            
    return el


def _find_first_object_tag(tags):
    """
    Finds the first object tag in the input tags dictionary.

    Args:
        tags: A dictionary of tags.

    Returns:
        The first object tag found, or None.
    """
    for name, tag in tags.items():
        tag_tuple = _convert(name, tag)
        tag_name = tag_tuple[0]
                
        if tag_name.lower() in OT._TAG_TO_CLASS:
            return tag_tuple

            
def create_element(element: List[Union[str, Dict[str, str], List]], parent=None) -> ET.Element:
    """
    Creates an XML element.
    
    Args:
        element: List of tag, attributes and children to feed into the element.
        parent: Parent element to append the new element to (Optional).
        
    Returns:
        An ElementTree element.
    """
    tag, attrs, children = element
    el = ET.SubElement(parent, tag) if parent is not None else ET.Element(tag)
    
    for k, v in attrs.items():
        el.set(k, v)

    if children:
        _create_nested_elements(el, children) 
        
    return el

        
def tree_from_tuples(*elements):
    """
    Creates an ElementTree from the input element definitions, and return a string for it.

    Args:
        elements: Variable length argument list of elements to add to the ElementTree.

    Returns:
        XML configuration
    """
    view = ET.Element('View')
    for element in elements:
        el = create_element(element)
        view.append(el)

    return view


def tree_to_string(tree, pretty=True):
    """
    """    
    if pretty:
        pp = _prettify(tree)
        return pp.replace('<?xml version="1.0" ?>\n', '')
    else:
        return ET.tostring(tree, encoding='unicode')
    

def convert_tags_description(tags: Dict[str, Any],
                             mapping: Optional[Dict[str, str]]=None) -> List[Union[str, Dict[str, str], List]]:
    """
    Convert tags into a structured format.

    Args:
        tags: A dictionary of tags.
        mapping: Optional mapping of tag name transformations.

    Returns:
        A list of the structured tags.
    """
    elements = []
    first_object_tag = _find_first_object_tag(tags)
        
    for name, tag in tags.items():
        el = _convert(name, tag)

        # Set `toName` property of the tag if tag name is in
        # CT._TAG_TO_CLASS and `toName` key is not in tag attributes
        # The value of `toName` key is set based on whether `name` of
        # tag is in the mapping dictionary or the `name` of the first
        # object tag if it's not in mapping
        if el[0].lower() in CT._TAG_TO_CLASS and ("toName" not in el[1] or el[1]["toName"] is None):
            if mapping and el[1].get("name") in mapping:
                el[1]["toName"] = mapping.get(el[1]["name"])                
            else:
                el[1]["toName"] = first_object_tag[1]["name"]
        
        elements.append(el)
    
    return elements



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class Workspace(pydantic_v1.BaseModel):
    id: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Unique ID of the workspace
    """

    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Workspace title
    """

    description: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Workspace description
    """

    is_public: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether the workspace is public or not
    """

    is_personal: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether the workspace is personal or not
    """

    is_archived: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether the workspace is archived or not
    """

    created_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Creation time of the workspace
    """

    updated_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Last updated time of the workspace
    """

    created_by: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    User ID of the workspace creator
    """

    color: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Workspace color
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .prompt_version import PromptVersion
from .refined_prompt_response_refinement_status import RefinedPromptResponseRefinementStatus


class RefinedPromptResponse(pydantic_v1.BaseModel):
    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Title of the refined prompt
    """

    reasoning: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Reasoning behind the refinement
    """

    prompt: str = pydantic_v1.Field()
    """
    The refined prompt text
    """

    refinement_job_id: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Unique identifier for the refinement job
    """

    refinement_status: typing.Optional[RefinedPromptResponseRefinementStatus] = pydantic_v1.Field(default=None)
    """
    Status of the refinement job
    """

    total_cost: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Total cost of the refinement job (in USD)
    """

    previous_version: typing.Optional[PromptVersion] = pydantic_v1.Field(default=None)
    """
    Previous version of the prompt
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import pydantic_v1
from ...core.request_options import RequestOptions
from ...types.inference_run import InferenceRun
from ...types.inference_run_created_by import InferenceRunCreatedBy
from ...types.inference_run_organization import InferenceRunOrganization
from ...types.inference_run_project_subset import InferenceRunProjectSubset
from ...types.inference_run_status import InferenceRunStatus
from .types.runs_list_request_project_subset import RunsListRequestProjectSubset

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RunsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        id: int,
        version_id: int,
        *,
        project: int,
        project_subset: RunsListRequestProjectSubset,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InferenceRun:
        """
        Get information (status, etadata, etc) about an existing inference run

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        project : int
            The ID of the project that this Interence Run makes predictions on

        project_subset : RunsListRequestProjectSubset
            Defines which tasks are operated on (e.g. HasGT will only operate on tasks with a ground truth annotation, but All will operate on all records)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InferenceRun
            Success

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.runs.list(
            id=1,
            version_id=1,
            project=1,
            project_subset="All",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/inference-runs",
            method="GET",
            params={"project": project, "project_subset": project_subset},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(InferenceRun, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        id: int,
        version_id: int,
        *,
        project: int,
        project_subset: InferenceRunProjectSubset,
        organization: typing.Optional[InferenceRunOrganization] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        created_by: typing.Optional[InferenceRunCreatedBy] = OMIT,
        status: typing.Optional[InferenceRunStatus] = OMIT,
        job_id: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        triggered_at: typing.Optional[dt.datetime] = OMIT,
        predictions_updated_at: typing.Optional[dt.datetime] = OMIT,
        completed_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InferenceRun:
        """
        Run a prompt inference.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        project : int

        project_subset : InferenceRunProjectSubset

        organization : typing.Optional[InferenceRunOrganization]

        model_version : typing.Optional[str]

        created_by : typing.Optional[InferenceRunCreatedBy]

        status : typing.Optional[InferenceRunStatus]

        job_id : typing.Optional[str]

        created_at : typing.Optional[dt.datetime]

        triggered_at : typing.Optional[dt.datetime]

        predictions_updated_at : typing.Optional[dt.datetime]

        completed_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InferenceRun


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.runs.create(
            id=1,
            version_id=1,
            project=1,
            project_subset="All",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/inference-runs",
            method="POST",
            json={
                "organization": organization,
                "project": project,
                "model_version": model_version,
                "created_by": created_by,
                "project_subset": project_subset,
                "status": status,
                "job_id": job_id,
                "created_at": created_at,
                "triggered_at": triggered_at,
                "predictions_updated_at": predictions_updated_at,
                "completed_at": completed_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(InferenceRun, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRunsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        id: int,
        version_id: int,
        *,
        project: int,
        project_subset: RunsListRequestProjectSubset,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InferenceRun:
        """
        Get information (status, etadata, etc) about an existing inference run

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        project : int
            The ID of the project that this Interence Run makes predictions on

        project_subset : RunsListRequestProjectSubset
            Defines which tasks are operated on (e.g. HasGT will only operate on tasks with a ground truth annotation, but All will operate on all records)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InferenceRun
            Success

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.runs.list(
            id=1,
            version_id=1,
            project=1,
            project_subset="All",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/inference-runs",
            method="GET",
            params={"project": project, "project_subset": project_subset},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(InferenceRun, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        id: int,
        version_id: int,
        *,
        project: int,
        project_subset: InferenceRunProjectSubset,
        organization: typing.Optional[InferenceRunOrganization] = OMIT,
        model_version: typing.Optional[str] = OMIT,
        created_by: typing.Optional[InferenceRunCreatedBy] = OMIT,
        status: typing.Optional[InferenceRunStatus] = OMIT,
        job_id: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        triggered_at: typing.Optional[dt.datetime] = OMIT,
        predictions_updated_at: typing.Optional[dt.datetime] = OMIT,
        completed_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InferenceRun:
        """
        Run a prompt inference.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        project : int

        project_subset : InferenceRunProjectSubset

        organization : typing.Optional[InferenceRunOrganization]

        model_version : typing.Optional[str]

        created_by : typing.Optional[InferenceRunCreatedBy]

        status : typing.Optional[InferenceRunStatus]

        job_id : typing.Optional[str]

        created_at : typing.Optional[dt.datetime]

        triggered_at : typing.Optional[dt.datetime]

        predictions_updated_at : typing.Optional[dt.datetime]

        completed_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InferenceRun


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.runs.create(
            id=1,
            version_id=1,
            project=1,
            project_subset="All",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/inference-runs",
            method="POST",
            json={
                "organization": organization,
                "project": project,
                "model_version": model_version,
                "created_by": created_by,
                "project_subset": project_subset,
                "status": status,
                "job_id": job_id,
                "created_at": created_at,
                "triggered_at": triggered_at,
                "predictions_updated_at": predictions_updated_at,
                "completed_at": completed_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(InferenceRun, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import pydantic_v1
from ...core.request_options import RequestOptions
from ...types.prompt_version import PromptVersion
from ...types.prompt_version_created_by import PromptVersionCreatedBy
from ...types.prompt_version_organization import PromptVersionOrganization
from ...types.prompt_version_provider import PromptVersionProvider
from ...types.refined_prompt_response import RefinedPromptResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[PromptVersion]:
        """
        Get a list of prompt versions.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[PromptVersion]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.list(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[PromptVersion], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        parent_model: typing.Optional[int] = OMIT,
        model_provider_connection: typing.Optional[int] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        provider: typing.Optional[PromptVersionProvider] = OMIT,
        provider_model_id: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptVersionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptVersionOrganization] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptVersion:
        """
        Create a new version of a prompt.

        Parameters
        ----------
        id : int
            Prompt ID

        title : typing.Optional[str]

        parent_model : typing.Optional[int]

        model_provider_connection : typing.Optional[int]

        prompt : typing.Optional[str]

        provider : typing.Optional[PromptVersionProvider]

        provider_model_id : typing.Optional[str]

        created_by : typing.Optional[PromptVersionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        organization : typing.Optional[PromptVersionOrganization]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.create(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions",
            method="POST",
            json={
                "title": title,
                "parent_model": parent_model,
                "model_provider_connection": model_provider_connection,
                "prompt": prompt,
                "provider": provider,
                "provider_model_id": provider_model_id,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": organization,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: int, version_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PromptVersion:
        """
        Get a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.get(
            id=1,
            version_id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, version_id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.delete(
            id=1,
            version_id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        version_id: int,
        *,
        title: typing.Optional[str] = OMIT,
        parent_model: typing.Optional[int] = OMIT,
        model_provider_connection: typing.Optional[int] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        provider: typing.Optional[PromptVersionProvider] = OMIT,
        provider_model_id: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptVersionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptVersionOrganization] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptVersion:
        """
        Update a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        title : typing.Optional[str]

        parent_model : typing.Optional[int]

        model_provider_connection : typing.Optional[int]

        prompt : typing.Optional[str]

        provider : typing.Optional[PromptVersionProvider]

        provider_model_id : typing.Optional[str]

        created_by : typing.Optional[PromptVersionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        organization : typing.Optional[PromptVersionOrganization]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.update(
            id=1,
            version_id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="PATCH",
            json={
                "title": title,
                "parent_model": parent_model,
                "model_provider_connection": model_provider_connection,
                "prompt": prompt,
                "provider": provider,
                "provider_model_id": provider_model_id,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": organization,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_refined_prompt(
        self,
        prompt_id: int,
        version_id: int,
        *,
        refinement_job_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefinedPromptResponse:
        """
        Get the refined prompt based on the `refinement_job_id`.

        Parameters
        ----------
        prompt_id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        refinement_job_id : str
            Refinement Job ID acquired from the `POST /api/prompts/{prompt_id}/versions/{version_id}/refine` endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefinedPromptResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.get_refined_prompt(
            prompt_id=1,
            version_id=1,
            refinement_job_id="refinement_job_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(prompt_id)}/versions/{jsonable_encoder(version_id)}/refine",
            method="GET",
            params={"refinement_job_id": refinement_job_id},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RefinedPromptResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refine_prompt(
        self,
        prompt_id: int,
        version_id: int,
        *,
        async_: typing.Optional[bool] = None,
        teacher_model_provider_connection_id: typing.Optional[int] = OMIT,
        teacher_model_name: typing.Optional[str] = OMIT,
        project_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefinedPromptResponse:
        """
        Refine a prompt version using a teacher model and save the refined prompt as a new version.

        Parameters
        ----------
        prompt_id : int
            Prompt ID

        version_id : int
            Base Prompt Version ID

        async_ : typing.Optional[bool]
            Run the refinement job asynchronously

        teacher_model_provider_connection_id : typing.Optional[int]
            Model Provider Connection ID to use to refine the prompt

        teacher_model_name : typing.Optional[str]
            Name of the model to use to refine the prompt

        project_id : typing.Optional[int]
            Project ID to target the refined prompt for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefinedPromptResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.versions.refine_prompt(
            prompt_id=1,
            version_id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(prompt_id)}/versions/{jsonable_encoder(version_id)}/refine",
            method="POST",
            params={"async": async_},
            json={
                "teacher_model_provider_connection_id": teacher_model_provider_connection_id,
                "teacher_model_name": teacher_model_name,
                "project_id": project_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RefinedPromptResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[PromptVersion]:
        """
        Get a list of prompt versions.

        Parameters
        ----------
        id : int
            Prompt ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[PromptVersion]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.list(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[PromptVersion], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        parent_model: typing.Optional[int] = OMIT,
        model_provider_connection: typing.Optional[int] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        provider: typing.Optional[PromptVersionProvider] = OMIT,
        provider_model_id: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptVersionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptVersionOrganization] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptVersion:
        """
        Create a new version of a prompt.

        Parameters
        ----------
        id : int
            Prompt ID

        title : typing.Optional[str]

        parent_model : typing.Optional[int]

        model_provider_connection : typing.Optional[int]

        prompt : typing.Optional[str]

        provider : typing.Optional[PromptVersionProvider]

        provider_model_id : typing.Optional[str]

        created_by : typing.Optional[PromptVersionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        organization : typing.Optional[PromptVersionOrganization]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.create(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions",
            method="POST",
            json={
                "title": title,
                "parent_model": parent_model,
                "model_provider_connection": model_provider_connection,
                "prompt": prompt,
                "provider": provider,
                "provider_model_id": provider_model_id,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": organization,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: int, version_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PromptVersion:
        """
        Get a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.get(
            id=1,
            version_id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: int, version_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.delete(
            id=1,
            version_id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        version_id: int,
        *,
        title: typing.Optional[str] = OMIT,
        parent_model: typing.Optional[int] = OMIT,
        model_provider_connection: typing.Optional[int] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        provider: typing.Optional[PromptVersionProvider] = OMIT,
        provider_model_id: typing.Optional[str] = OMIT,
        created_by: typing.Optional[PromptVersionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        organization: typing.Optional[PromptVersionOrganization] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PromptVersion:
        """
        Update a prompt version by ID.

        Parameters
        ----------
        id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        title : typing.Optional[str]

        parent_model : typing.Optional[int]

        model_provider_connection : typing.Optional[int]

        prompt : typing.Optional[str]

        provider : typing.Optional[PromptVersionProvider]

        provider_model_id : typing.Optional[str]

        created_by : typing.Optional[PromptVersionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        organization : typing.Optional[PromptVersionOrganization]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PromptVersion


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.update(
            id=1,
            version_id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}",
            method="PATCH",
            json={
                "title": title,
                "parent_model": parent_model,
                "model_provider_connection": model_provider_connection,
                "prompt": prompt,
                "provider": provider,
                "provider_model_id": provider_model_id,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "organization": organization,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(PromptVersion, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_refined_prompt(
        self,
        prompt_id: int,
        version_id: int,
        *,
        refinement_job_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefinedPromptResponse:
        """
        Get the refined prompt based on the `refinement_job_id`.

        Parameters
        ----------
        prompt_id : int
            Prompt ID

        version_id : int
            Prompt Version ID

        refinement_job_id : str
            Refinement Job ID acquired from the `POST /api/prompts/{prompt_id}/versions/{version_id}/refine` endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefinedPromptResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.get_refined_prompt(
            prompt_id=1,
            version_id=1,
            refinement_job_id="refinement_job_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(prompt_id)}/versions/{jsonable_encoder(version_id)}/refine",
            method="GET",
            params={"refinement_job_id": refinement_job_id},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RefinedPromptResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refine_prompt(
        self,
        prompt_id: int,
        version_id: int,
        *,
        async_: typing.Optional[bool] = None,
        teacher_model_provider_connection_id: typing.Optional[int] = OMIT,
        teacher_model_name: typing.Optional[str] = OMIT,
        project_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RefinedPromptResponse:
        """
        Refine a prompt version using a teacher model and save the refined prompt as a new version.

        Parameters
        ----------
        prompt_id : int
            Prompt ID

        version_id : int
            Base Prompt Version ID

        async_ : typing.Optional[bool]
            Run the refinement job asynchronously

        teacher_model_provider_connection_id : typing.Optional[int]
            Model Provider Connection ID to use to refine the prompt

        teacher_model_name : typing.Optional[str]
            Name of the model to use to refine the prompt

        project_id : typing.Optional[int]
            Project ID to target the refined prompt for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefinedPromptResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.prompts.versions.refine_prompt(
            prompt_id=1,
            version_id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/prompts/{jsonable_encoder(prompt_id)}/versions/{jsonable_encoder(version_id)}/refine",
            method="POST",
            params={"async": async_},
            json={
                "teacher_model_provider_connection_id": teacher_model_provider_connection_id,
                "teacher_model_name": teacher_model_name,
                "project_id": project_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(RefinedPromptResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class PromptsBatchPredictionsRequestResultsItem(pydantic_v1.BaseModel):
    task_id: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Task ID to associate the prediction with
    """

    output: typing.Optional[typing.Dict[str, typing.Any]] = pydantic_v1.Field(default=None)
    """
    Prediction output that contains keys from labeling config. Each key must be a valid control tag name from the labeling config. For example, given the output: `json {"sentiment": "positive"} ` it will be converted to the internal LS annotation format: `json { "value": { "choices": ["positive"] }, "from_name": "label", "to_name": "", ... } `
    """

    prompt_tokens: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Number of tokens in the prompt
    """

    completion_tokens: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Number of tokens in the completion
    """

    prompt_cost_usd: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Cost of the prompt (in USD)
    """

    completion_cost_usd: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Cost of the completion (in USD)
    """

    total_cost_usd: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Total cost of the inference (in USD)
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .serialization_option import SerializationOption


class SerializationOptions(pydantic_v1.BaseModel):
    drafts: typing.Optional[SerializationOption] = None
    predictions: typing.Optional[SerializationOption] = None
    include_annotation_history: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Include annotation history
    """

    annotations_completed_by: typing.Optional[SerializationOption] = pydantic_v1.Field(
        alias="annotations__completed_by", default=None
    )
    interpolate_key_frames: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Interpolate video key frames
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



import argparse
import datetime
import hashlib
import io
import logging
import math
import os
import re
import shutil
import urllib
import wave
from collections import defaultdict
from copy import deepcopy
from operator import itemgetter
from urllib.parse import urlparse

import numpy as np
import requests
from PIL import Image
from lxml import etree
from nltk.tokenize.treebank import TreebankWordTokenizer

from label_studio_sdk._extensions.label_studio_tools.core.utils.params import get_env

logger = logging.getLogger(__name__)

_LABEL_TAGS = {"Label", "Choice"}
_NOT_CONTROL_TAGS = {
    "Filter",
}
LOCAL_FILES_DOCUMENT_ROOT = get_env(
    "LOCAL_FILES_DOCUMENT_ROOT", default=os.path.abspath(os.sep)
)

TreebankWordTokenizer.PUNCTUATION = [
    (re.compile(r"([:,])([^\d])"), r" \1 \2"),
    (re.compile(r"([:,])$"), r" \1 "),
    (re.compile(r"\.\.\."), r" ... "),
    (re.compile(r"[;@#$/%&]"), r" \g<0> "),
    (
        re.compile(r'([^\.])(\.)([\]\)}>"\']*)\s*$'),
        r"\1 \2\3 ",
    ),  # Handles the final period.
    (re.compile(r"[?!]"), r" \g<0> "),
    (re.compile(r"([^'])' "), r"\1 ' "),
]


class ExpandFullPath(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, os.path.abspath(os.path.expanduser(values)))


def tokenize(text):
    tok_start = 0
    out = []
    for tok in text.split():
        if len(tok):
            out.append((tok, tok_start))
            tok_start += len(tok) + 1
        else:
            tok_start += 1
    return out


def create_tokens_and_tags(text, spans):
    # tokens_and_idx = tokenize(text) # This function doesn't work properly if text contains multiple whitespaces...
    token_index_tuples = [
        token for token in TreebankWordTokenizer().span_tokenize(text)
    ]
    tokens_and_idx = [(text[start:end], start) for start, end in token_index_tuples]
    if spans and all(
        [
            span.get("start") is not None and span.get("end") is not None
            for span in spans
        ]
    ):
        spans = list(sorted(spans, key=itemgetter("start")))
        span = spans.pop(0)
        span_start = span["start"]
        span_end = span["end"] - 1
        prefix = "B-"
        tokens, tags = [], []
        for token, token_start in tokens_and_idx:
            tokens.append(token)
            token_end = (
                token_start + len(token) - 1
            )  # "- 1" - This substraction is wrong. token already uses the index E.g. "Hello" is 0-4
            token_start_ind = token_start  # It seems like the token start is too early.. for whichever reason

            # if for some reason end of span is missed.. pop the new span (Which is quite probable due to this method)
            # Attention it seems like span['end'] is the index of first char afterwards. In case the whitespace is part of the
            # labell we need to subtract one. Otherwise next token won't trigger the span update.. only the token after next..
            if token_start_ind > span_end:
                while spans:
                    span = spans.pop(0)
                    span_start = span["start"]
                    span_end = span["end"] - 1
                    prefix = "B-"
                    if token_start <= span_end:
                        break
            # Add tag "O" for spans that:
            # - are empty
            # - span start has passed over token_end
            # - do not have any label (None or empty list)
            if not span or token_end < span_start or not span.get("labels"):
                tags.append("O")
            elif span_start <= token_end and span_end >= token_start_ind:
                tags.append(prefix + span["labels"][0])
                prefix = "I-"
            else:
                tags.append("O")
    else:
        tokens = [token for token, _ in tokens_and_idx]
        tags = ["O"] * len(tokens)

    return tokens, tags


def _get_upload_dir(project_dir=None, upload_dir=None):
    """Return either upload_dir, or path by LS_UPLOAD_DIR, or project_dir/upload"""
    if upload_dir:
        return upload_dir
    upload_dir = os.environ.get("LS_UPLOAD_DIR")
    if not upload_dir and project_dir:
        upload_dir = os.path.join(project_dir, "upload")
        if not os.path.exists(upload_dir):
            upload_dir = None
    if not upload_dir:
        raise FileNotFoundError(
            "Can't find upload dir: either LS_UPLOAD_DIR or project should be passed to converter"
        )
    return upload_dir


def download(
    url,
    output_dir,
    filename=None,
    project_dir=None,
    return_relative_path=False,
    upload_dir=None,
    download_resources=True,
):
    is_local_file = url.startswith("/data/") and "?d=" in url
    is_uploaded_file = url.startswith("/data/upload")

    if is_uploaded_file:
        upload_dir = _get_upload_dir(project_dir, upload_dir)
        filename = urllib.parse.unquote(url.replace("/data/upload/", ""))
        filepath = os.path.join(upload_dir, filename)
        logger.debug(
            f"Copy {filepath} to {output_dir}".format(
                filepath=filepath, output_dir=output_dir
            )
        )
        if download_resources:
            shutil.copy(filepath, output_dir)
        if return_relative_path:
            return os.path.join(
                os.path.basename(output_dir), os.path.basename(filename)
            )
        return filepath

    if is_local_file:
        filename, dir_path = url.split("/data/", 1)[-1].split("?d=")
        dir_path = str(urllib.parse.unquote(dir_path))
        filepath = os.path.join(LOCAL_FILES_DOCUMENT_ROOT, dir_path)
        if not os.path.exists(filepath):
            raise FileNotFoundError(filepath)
        if download_resources:
            shutil.copy(filepath, output_dir)
        return filepath

    if filename is None:
        basename, ext = os.path.splitext(os.path.basename(urlparse(url).path))
        filename = f"{basename}{ext}"
        filepath = os.path.join(output_dir, filename)
        if os.path.exists(filepath):
            filename = (
                basename
                + "_"
                + hashlib.md5(
                    url.encode() + str(datetime.datetime.now().timestamp()).encode()
                ).hexdigest()[:4]
                + ext
            )

    filepath = os.path.join(output_dir, filename)
    if not os.path.exists(filepath):
        logger.info("Download {url} to {filepath}".format(url=url, filepath=filepath))
        if download_resources:
            r = requests.get(url)
            r.raise_for_status()
            with io.open(filepath, mode="wb") as fout:
                fout.write(r.content)
    if return_relative_path:
        return os.path.join(os.path.basename(output_dir), os.path.basename(filename))
    return filepath


def get_image_size(image_path):
    return Image.open(image_path).size


def get_image_size_and_channels(image_path):
    i = Image.open(image_path)
    w, h = i.size
    c = len(i.getbands())
    return w, h, c


def get_audio_duration(audio_path):
    with wave.open(audio_path, mode="r") as f:
        return f.getnframes() / float(f.getframerate())


def ensure_dir(dir_path):
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)


def parse_config(config_string):
    """
    :param config_string: Label config string
    :return: structured config of the form:
    {
        "<ControlTag>.name": {
            "type": "ControlTag",
            "to_name": ["<ObjectTag1>.name", "<ObjectTag2>.name"],
            "inputs: [
                {"type": "ObjectTag1", "value": "<ObjectTag1>.value"},
                {"type": "ObjectTag2", "value": "<ObjectTag2>.value"}
            ],
            "labels": ["Label1", "Label2", "Label3"] // taken from "alias" if exists or "value"
    }
    """
    if not config_string:
        return {}

    def _is_input_tag(tag):
        return tag.attrib.get("name") and tag.attrib.get("value")

    def _is_output_tag(tag):
        return (
            tag.attrib.get("name")
            and tag.attrib.get("toName")
            and tag.tag not in _NOT_CONTROL_TAGS
        )

    def _get_parent_output_tag_name(tag, outputs):
        # Find parental <Choices> tag for nested tags like <Choices><View><View><Choice>...
        parent = tag
        while True:
            parent = parent.getparent()
            if parent is None:
                return
            name = parent.attrib.get("name")
            if name in outputs:
                return name

    try:
        xml_tree = etree.fromstring(config_string)
    except etree.XMLSyntaxError as e:
        raise ValueError(str(e))

    inputs, outputs, labels = {}, {}, defaultdict(dict)
    for tag in xml_tree.iter():
        if _is_output_tag(tag):
            tag_info = {"type": tag.tag, "to_name": tag.attrib["toName"].split(",")}
            # Grab conditionals if any
            conditionals = {}
            if tag.attrib.get("perRegion") == "true":
                if tag.attrib.get("whenTagName"):
                    conditionals = {"type": "tag", "name": tag.attrib["whenTagName"]}
                elif tag.attrib.get("whenLabelValue"):
                    conditionals = {
                        "type": "label",
                        "name": tag.attrib["whenLabelValue"],
                    }
                elif tag.attrib.get("whenChoiceValue"):
                    conditionals = {
                        "type": "choice",
                        "name": tag.attrib["whenChoiceValue"],
                    }
            if conditionals:
                tag_info["conditionals"] = conditionals
            outputs[tag.attrib["name"]] = tag_info
        elif _is_input_tag(tag):
            inputs[tag.attrib["name"]] = {
                "type": tag.tag,
                "value": tag.attrib["value"].lstrip("$"),
            }
        if tag.tag not in _LABEL_TAGS:
            continue
        parent_name = _get_parent_output_tag_name(tag, outputs)
        if parent_name is not None:
            actual_value = tag.attrib.get("alias") or tag.attrib.get("value")
            if not actual_value:
                logger.debug(
                    'Inspecting tag {tag_name}... found no "value" or "alias" attributes.'.format(
                        tag_name=etree.tostring(tag, encoding="unicode").strip()[:50]
                    )
                )
            else:
                labels[parent_name][actual_value] = dict(tag.attrib)
    for output_tag, tag_info in outputs.items():
        tag_info["inputs"] = []
        for input_tag_name in tag_info["to_name"]:
            if input_tag_name not in inputs:
                logger.debug(
                    f"to_name={input_tag_name} is specified for output tag name={output_tag}, "
                    "but we can't find it among input tags"
                )
                continue
            tag_info["inputs"].append(inputs[input_tag_name])
        tag_info["labels"] = list(labels[output_tag])
        tag_info["labels_attrs"] = labels[output_tag]
    return outputs


def get_polygon_area(x, y):
    """https://en.wikipedia.org/wiki/Shoelace_formula"""

    assert len(x) == len(y)

    return float(0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1))))


def get_polygon_bounding_box(x, y):
    assert len(x) == len(y)

    x1, y1, x2, y2 = min(x), min(y), max(x), max(y)
    return [x1, y1, x2 - x1, y2 - y1]


def get_annotator(item, default=None, int_id=False):
    """Get annotator id or email from annotation"""
    annotator = item["completed_by"]
    if isinstance(annotator, dict):
        annotator = annotator.get("email", default)
        return annotator

    if isinstance(annotator, int) and int_id:
        return annotator

    return str(annotator)


def get_json_root_type(filename):
    char = "x"
    with open(filename, "r", encoding="utf-8") as f:
        # Read the file character by character
        while char != "":
            char = f.read(1)

            # Skip any whitespace
            if char.isspace():
                continue

            # If the first non-whitespace character is '{', it's a dict
            if char == "{":
                return "dict"

            # If the first non-whitespace character is '[', it's an array
            if char == "[":
                return "list"

            # If neither, the JSON file is invalid
            return "invalid"

    # If the file is empty, return "empty"
    return "empty"


def prettify_result(v):
    """
    :param v: list of regions or results
    :return: label name as is if there is only 1 item in result `v`, else list of label names
    """
    out = []
    tag_type = None
    for i in v:
        j = deepcopy(i)
        tag_type = j.pop("type")
        if tag_type == "Choices" and len(j["choices"]) == 1:
            out.append(j["choices"][0])
        elif tag_type == "TextArea" and len(j["text"]) == 1:
            out.append(j["text"][0])
        else:
            out.append(j)
    return out[0] if tag_type in ("Choices", "TextArea") and len(out) == 1 else out


def convert_annotation_to_yolo(label):
    """
    Convert LS annotation to Yolo format.

    Args:
        label (dict): Dictionary containing annotation information including:
            - width (float): Width of the object.
            - height (float): Height of the object.
            - x (float): X-coordinate of the top-left corner of the object.
            - y (float): Y-coordinate of the top-left corner of the object.

    Returns:
        tuple or None: If the conversion is successful, returns a tuple (x, y, w, h) representing
        the coordinates and dimensions of the object in Yolo format, where (x, y) are the center
        coordinates of the object, and (w, h) are the width and height of the object respectively.
    """

    if not ("x" in label and "y" in label and "width" in label and "height" in label):
        return None

    w = label["width"]
    h = label["height"]

    x = (label["x"] + w / 2) / 100
    y = (label["y"] + h / 2) / 100
    w = w / 100
    h = h / 100

    return x, y, w, h


def convert_annotation_to_yolo_obb(label, normalize=True):
    """
    Convert LS annotation to Yolo OBB format.

    Args:
        label (dict): Dictionary containing annotation information including:
            - original_width (int): Original width of the image.
            - original_height (int): Original height of the image.
            - x (float): X-coordinate of the top-left corner of the object in percentage of the original width.
            - y (float): Y-coordinate of the top-left corner of the object in percentage of the original height.
            - width (float): Width of the object in percentage of the original width.
            - height (float): Height of the object in percentage of the original height.
            - rotation (float, optional): Rotation angle of the object in degrees (default is 0).
        normalize (bool, optional): Whether to normalize the coordinates to the range [0, 1] (default is True).

    Returns:
        list of tuple or None: List of tuples containing the coordinates of the object in Yolo OBB format.
            Each tuple represents a corner of the bounding box in the order:
            (top-left, top-right, bottom-right, bottom-left).
    """

    if not (
        "original_width" in label
        and "original_height" in label
        and "x" in label
        and "y" in label
        and "width" in label
        and "height" in label
        and "rotation" in label
    ):
        return None

    org_width, org_height = label["original_width"], label["original_height"]
    x = label["x"] / 100 * org_width
    y = label["y"] / 100 * org_height
    w = label["width"] / 100 * org_width
    h = label["height"] / 100 * org_height

    rotation = math.radians(label.get("rotation", 0))
    cos, sin = math.cos(rotation), math.sin(rotation)

    coords = [
        (x, y),
        (x + w * cos, y + w * sin),
        (x + w * cos - h * sin, y + w * sin + h * cos),
        (x - h * sin, y + h * cos),
    ]

    # Normalize coordinates
    if normalize:
        return [(coord[0] / org_width, coord[1] / org_height) for coord in coords]
    else:
        return coords


def convert_yolo_obb_to_annotation(xyxyxyxy, original_width, original_height):
    """
    Convert YOLO Oriented Bounding Box (OBB) format to Label Studio format.

    Args:
        xyxyxyxy (list): List of 8 float values representing the absolute pixel coordinates
                         of the OBB in the format [x1, y1, x2, y2, x3, y3, x4, y4].
        original_width (int): Original width of the image.
        original_height (int): Original height of the image.

    Returns:
        dict: Dictionary containing the converted bounding box with the following keys:
              - x: X-coordinate of the top-left corner of the bounding box in percentage.
              - y: Y-coordinate of the top-left corner of the bounding box in percentage.
              - width: Width of the bounding box in percentage.
              - height: Height of the bounding box in percentage.
              - rotation: Rotation angle of the bounding box in degrees.
    """
    # Reshape the coordinates into a 4x2 matrix
    coords = np.array(xyxyxyxy, dtype=np.float64).reshape((4, 2))

    # Calculate the center of the bounding box
    center_x = np.mean(coords[:, 0])
    center_y = np.mean(coords[:, 1])

    # Calculate the width and height of the bounding box
    width = np.linalg.norm(coords[0] - coords[1])
    height = np.linalg.norm(coords[0] - coords[3])

    # Calculate the rotation angle
    dx = coords[1, 0] - coords[0, 0]
    dy = coords[1, 1] - coords[0, 1]
    r = np.degrees(np.arctan2(dy, dx))

    # Find the top-left corner (x, y)
    top_left_x = (
        center_x
        - (width / 2) * np.cos(np.radians(r))
        + (height / 2) * np.sin(np.radians(r))
    )
    top_left_y = (
        center_y
        - (width / 2) * np.sin(np.radians(r))
        - (height / 2) * np.cos(np.radians(r))
    )

    # Normalize the values
    x = (top_left_x / original_width) * 100
    y = (top_left_y / original_height) * 100
    width = (width / original_width) * 100
    height = (height / original_height) * 100

    # Create the dictionary for Label Studio
    return {
        "x": x,
        "y": y,
        "width": width,
        "height": height,
        "rotation": r,
        "original_width": original_width,
        "original_height": original_height,
    }



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..types.model_provider_connection import ModelProviderConnection
from ..types.model_provider_connection_created_by import ModelProviderConnectionCreatedBy
from ..types.model_provider_connection_organization import ModelProviderConnectionOrganization
from ..types.model_provider_connection_provider import ModelProviderConnectionProvider
from ..types.model_provider_connection_scope import ModelProviderConnectionScope

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ModelProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ModelProviderConnection:
        """
        Create a new model provider connection.

        Parameters
        ----------
        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.create(
            provider="OpenAI",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="POST",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": organization,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ModelProviderConnection, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncModelProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> ModelProviderConnection:
        """
        Create a new model provider connection.

        Parameters
        ----------
        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.model_providers.create(
            provider="OpenAI",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="POST",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": organization,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(ModelProviderConnection, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import pydantic_v1
from ...core.request_options import RequestOptions
from .types.members_create_response import MembersCreateResponse
from .types.members_list_response_item import MembersListResponseItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MembersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[MembersListResponseItem]:
        """
        List all workspace memberships for a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[MembersListResponseItem]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.members.list(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[MembersListResponseItem], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self, id: int, *, user: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> MembersCreateResponse:
        """
        Create a new workspace membership. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        user : typing.Optional[int]
            User ID of the workspace member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MembersCreateResponse


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.members.create(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships",
            method="POST",
            json={"user": user},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(MembersCreateResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, id: int, *, user: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a specific workspace membership. You will need to provide the workspace ID and the user ID. You can find this using [List workspace memberships](list).

        Parameters
        ----------
        id : int
            Workspace ID

        user : typing.Optional[int]
            User ID of the workspace member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.members.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships",
            method="DELETE",
            json={"user": user},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncMembersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[MembersListResponseItem]:
        """
        List all workspace memberships for a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[MembersListResponseItem]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.workspaces.members.list(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[MembersListResponseItem], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self, id: int, *, user: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> MembersCreateResponse:
        """
        Create a new workspace membership. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        user : typing.Optional[int]
            User ID of the workspace member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MembersCreateResponse


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.workspaces.members.create(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships",
            method="POST",
            json={"user": user},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(MembersCreateResponse, _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: int, *, user: typing.Optional[int] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a specific workspace membership. You will need to provide the workspace ID and the user ID. You can find this using [List workspace memberships](list).

        Parameters
        ----------
        id : int
            Workspace ID

        user : typing.Optional[int]
            User ID of the workspace member

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.workspaces.members.delete(
            id=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}/memberships",
            method="DELETE",
            json={"user": user},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from .azure.client import AsyncAzureClient, AzureClient
from .gcs.client import AsyncGcsClient, GcsClient
from .local.client import AsyncLocalClient, LocalClient
from .redis.client import AsyncRedisClient, RedisClient
from .s3.client import AsyncS3Client, S3Client
from .s3s.client import AsyncS3SClient, S3SClient
from .types.import_storage_list_types_response_item import ImportStorageListTypesResponseItem


class ImportStorageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.azure = AzureClient(client_wrapper=self._client_wrapper)
        self.gcs = GcsClient(client_wrapper=self._client_wrapper)
        self.local = LocalClient(client_wrapper=self._client_wrapper)
        self.redis = RedisClient(client_wrapper=self._client_wrapper)
        self.s3 = S3Client(client_wrapper=self._client_wrapper)
        self.s3s = S3SClient(client_wrapper=self._client_wrapper)

    def list_types(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ImportStorageListTypesResponseItem]:
        """
        Retrieve a list of the import storages types.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ImportStorageListTypesResponseItem]


        Examples
        --------
        from label_studio_sdk.client import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.import_storage.list_types()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/types", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[ImportStorageListTypesResponseItem], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncImportStorageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.azure = AsyncAzureClient(client_wrapper=self._client_wrapper)
        self.gcs = AsyncGcsClient(client_wrapper=self._client_wrapper)
        self.local = AsyncLocalClient(client_wrapper=self._client_wrapper)
        self.redis = AsyncRedisClient(client_wrapper=self._client_wrapper)
        self.s3 = AsyncS3Client(client_wrapper=self._client_wrapper)
        self.s3s = AsyncS3SClient(client_wrapper=self._client_wrapper)

    async def list_types(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ImportStorageListTypesResponseItem]:
        """
        Retrieve a list of the import storages types.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ImportStorageListTypesResponseItem]


        Examples
        --------
        from label_studio_sdk.client import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )
        await client.import_storage.list_types()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/types", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return pydantic_v1.parse_obj_as(typing.List[ImportStorageListTypesResponseItem], _response.json())  # type: ignore
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .prompt_created_by import PromptCreatedBy
from .prompt_organization import PromptOrganization


class Prompt(pydantic_v1.BaseModel):
    title: str = pydantic_v1.Field()
    """
    Title of the prompt
    """

    description: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Description of the prompt
    """

    created_by: typing.Optional[PromptCreatedBy] = pydantic_v1.Field(default=None)
    """
    User ID of the creator of the prompt
    """

    created_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Date and time the prompt was created
    """

    updated_at: typing.Optional[dt.datetime] = pydantic_v1.Field(default=None)
    """
    Date and time the prompt was last updated
    """

    organization: typing.Optional[PromptOrganization] = pydantic_v1.Field(default=None)
    """
    Organization ID of the prompt
    """

    input_fields: typing.List[str] = pydantic_v1.Field()
    """
    List of input fields
    """

    output_classes: typing.List[str] = pydantic_v1.Field()
    """
    List of output classes
    """

    associated_projects: typing.Optional[typing.List[int]] = pydantic_v1.Field(default=None)
    """
    List of associated projects IDs
    """

    skill_name: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Name of the skill
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import ProjectsCreateResponse, ProjectsImportTasksResponse, ProjectsListResponse, ProjectsUpdateResponse
from . import exports

__all__ = [
    "ProjectsCreateResponse",
    "ProjectsImportTasksResponse",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
    "exports",
]
